# This file is part of Carbon-Slack.
#
# Carbon-Slack is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Carbon-Slack is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Carbon-Slack.  If not, see <https://www.gnu.org/licenses/>.

import sys
import click
import carbon_slack.config
import carbon_slack.relay
import carbon_slack.slack
import time
import datetime

@click.command()
def init():
    """Print a sample configuration file"""
    carbon_slack.config.serialized_sample()

@click.command()
@click.option('--config-file', '-c', help='Alternative config YAML')
def relay(config_file=None):
    """Start up a relay that reads metrics from plaintext Slack, and pushes to plaintext Carbon"""
    cfg = carbon_slack.config.load(config_file)

    client = carbon_slack.relay.Relay(cfg)
    try:
        while True:
            client.run()
            time.sleep(10)
    except KeyboardInterrupt:
        if client is not None:
            try:
                client.close()
            finally:
                print "attempted to close relay"

    sys.stderr.write("\nExiting\n")
    sys.exit(0)

@click.command()
@click.option('--config-file', '-c', help='Alternative config YAML')
def recv(config_file=None):
    """Receive metric messages coming from Slack plaintext messages.

    This is intended as a way to test Slack message sending.
    """
    cfg = carbon_slack.config.load(config_file)

    recv = carbon_slack.slack.Receiver(cfg)
    messages = recv.get_messages()
    for m in messages:
        username = recv.find_user(m['user'])
        m['username'] = username
        m['datestamp'] = datetime.fromtimestamp(m['ts'])
        print "%(datestamp)s  %(username)s: %(text)s" % m

@click.command()
@click.argument('metric_name')
@click.argument('metric_value')
@click.option('--config-file', '-c', help='Alternative config YAML')
def send(args, config_file=None):
    """Command-line sender for metrics to Slack (using plaintext messages).

    This could be useful for testing a Slack connection, or for calling from 
    a shell script or similar, simple service.
    """
    cfg = carbon_slack.config.load(config_file)

    sender = carbon_slack.slack.Sender(cfg)
    now = int(time.time())
    sender.send("%s %s %s" % (args.metric_name, args.metric_value, now))




#!/usr/bin/env python

# This file is part of Carbon-Slack.

# Carbon-Slack is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Carbon-Slack is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Carbon-Slack.  If not, see <https://www.gnu.org/licenses/>.

import carbon_slack.config as conf
from carbon_slack.slack import Sender
import sys

config = {
	conf.TOKEN: sys.argv[1],
	conf.CHANNEL: sys.argv[2] if len(sys.argv) > 2 else None
}

parts = []
while True:
	send = Sender(config)

	part = sys.stdin.readline().rstrip()
	if part == '.':
		send.send(parts)
		parts = []
	else:
		parts.append(part)

	


# This file is part of Carbon-Slack.
#
# Carbon-Slack is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Carbon-Slack is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Carbon-Slack.  If not, see <https://www.gnu.org/licenses/>.

import socket

class PlaintextSender(object):
    def __init__(self, config):
        """Initialize a new Carbon sender that will use the plaintext protocol for sending metrics"""
        self.config = config

    def close(self):
        """If a socket is open for sending data at the time when the user kills this script, try to close it 
        gracefully.
        """
        if self.sock is not None:
            self.sock.close()

    def send_metrics(self, metrics):
        """Serialize the metrics dict (of the form k:v without timestamps) along with a generated timestamp into a
        multi-line payload. Then, send it over a new socket to the Carbon daemon.
        """
        self.sock = socket.socket()
        try:
            self.sock.connect( (self.config.carbon_server, self.config.carbon_port) )
            
            output = "\n".join(metrics)
            print( "Sending:\n%s" % output)
            self.sock.send(output)
        except socket.error:
            raise SystemExit("Couldn't connect to %(server)s on port %(port)d, is carbon-cache.py running?" % 
                { 'server':self.config.carbon_server, 'port':self.config.carbon_port })

        finally:
            self.sock.close()
            self.sock = None




# This file is part of Carbon-Slack.
#
# Carbon-Slack is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Carbon-Slack is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Carbon-Slack.  If not, see <https://www.gnu.org/licenses/>.

from ruamel.yaml import YAML
import getpass
import subprocess
import os
import sys

CONFIG_PATH = '/etc/carbon-slack.yml'

TOKEN = 'token'
CHANNEL = 'channel'
CARBON_SERVER='carbon-server'
CARBON_PORT='carbon-port'

class Config(object):
    def __init__(self, data):
        """Create a configuration instance suitable for use with the Carbon-Slack relay (and associated commands).

        NOTE: This could be created from an external YAML (or dict) source, and doesn't have to be read from the 
        CONFIG_PATH (/etc/carbon-slack.yml by default).
        """
        self.token = data[TOKEN]
        self.channel = data[CHANNEL]
        self.carbon_server = data[CARBON_SERVER]
        self.carbon_port = data[CARBON_PORT]

def serialized_sample():
    """Serialize a sample configuration to STDOUT"""
    return YAML().dump({
        TOKEN: 'xoxp-1234567890-yadda-yadda',
        CHANNEL: 'metrics',
        CARBON_SERVER: '127.0.0.1',
        CARBON_PORT: 2023
    }, sys.stdout)

def load(config_file=None):
    """Load a Carbon-Slack relay configuration from a YAML file (by default, use /etc/carbon-slack.yml)"""
    config_path = config_file or CONFIG_PATH
    data = {}

    with open(config_path) as f:
        data = YAML().load(f)

    return Config(data)





# This file is part of Carbon-Slack.
#
# Carbon-Slack is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Carbon-Slack is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Carbon-Slack.  If not, see <https://www.gnu.org/licenses/>.

import slackclient as client
import time

def find_channel_id(slack, name):
    """Allow a user to configure this system using a user-friendly channel name.
    The network handlers will use this method to map that to a channel ID.
    """
    result = slack.api_call("channels.list", exclude_members=True)
    for channel in result['channels']:
        if channel['name'] == name:
            return channel['id']

        prev_names = channel.get('previous_names')
        if prev_names is not None and name in prev_names:
            return channel['id']

    return None

class Receiver(object):
    def __init__(self, config, last_message=0):
        """Construct a new Slack channel message receiver using the given configuration dict and an
        optional last-message timestamp (unix seconds).
        """
        self.config = config
        self.slack = client.SlackClient(config.token)

        self.channel_id = find_channel_id(self.slack, config.channel)
        if self.channel_id is not None:
            self.slack.api_call('channels.join', channel=self.channel_id)

        self.last_message = last_message

    def list_channels(self):
        """List the available Slack channels."""
        result = self.slack.api_call("channels.list")
        return [channel for channel in result['channels']]

    def find_user(self, user_id):
        """Lookup the user information for the given user id. Things like username will be available this way.
        """
        result = self.slack.api_call('users.info', user=user_id, include_locale=False)
        return result.get('user')

    def get_last_seen_message(self):
        """Return the last-seen message after zero or more messages have been processed. This Receiver
        will keep track of the last-seen timestamp as it receives messages, such that the calling
        application can store this value and use it when restarting the script later.
        """
        return self.last_message

    def get_messages(self):
        """Receive any available messages on the Slack channel, and update the last-message timestamp
        appropriately before returning the array of message objects.
        """
        result = self.slack.api_call('channels.history', channel=self.channel_id, oldest=self.last_message)
        self.last_message=int(time.time())

        messages = result.get('messages') or []
        messages = messages[::-1]

        return messages

    def ack_messages(self, messages_ts):
        """Acknowledge that processing is complete for an array of messages (given by their 'ts' values in messages_ts).
        This acknowledgement happens by way of deleting the messages out of the channel to prevent duplicate processing.
        """
        if messages_ts is not None:
            for ts in messages_ts:
                self.slack.api_call('chat.delete', channel=self.channel_id, ts=ts)

class Sender(object):
    def __init__(self, config):
        """Construct a new Slack channel message sender using the given configuration dict to specify a token and 
        channel name.
        """
        self.config = config
        self.slack = client.SlackClient(config.token)

        self.channel_id = find_channel_id(self.slack, config.channel)
        if self.channel_id is not None:
            self.slack.api_call('channels.join', channel=self.channel_id)


    def list_channels(self):
        """List the available Slack channels."""
        result = self.slack.api_call("channels.list")
        return [channel for channel in result['channels']]

    def find_user(self, user_id):
        """Lookup the user information for the given user id. Things like username will be available this way.
        """
        result = self.slack.api_call('users.info', user=user_id, include_locale=False)
        return result.get('user')

    def send_metrics(self, metrics):
        """Serialize the given metrics dict (assumed to be key:value pairs without timestamps) to triplets in a 
        multi-line message, using a timestamp generated in this method. Once the message is formatted, send it to 
        the configured Slack channel.
        """
        now = int(time.time())

        lines = []
        for k in metrics.keys():
            v = metrics[k]
            lines.append("%s %s %s" % (k, v, now))

        self.send("\n".join(lines))

    def send(self, message):
        """Send a simple string message to the configured Slack channel."""
        return self.slack.api_call('chat.postMessage', channel=self.channel_id, text=message)



# This file is part of Carbon-Slack.
#
# Carbon-Slack is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Carbon-Slack is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Carbon-Slack.  If not, see <https://www.gnu.org/licenses/>.

import time
import socket
import carbon_slack.slack
import carbon_slack.carbon

class Relay(object):
    def __init__(self, config, last_message=0):
        """Initialize the Relay using a config dict and an optional last-message timestamp (in unix seconds).
        """
        self.recv = carbon_slack.slack.Receiver(config, last_message)
        self.send = carbon_slack.carbon.PlaintextSender(config)
        self.config = config

    def close(self):
        """If a socket is open in the sender at the time when the user kills this script, try to close it 
        gracefully.
        """
        self.send.close()

    def run(self):
        """Receive any available messages from the Slack channel and assume they are metrics triplets (key, value, tstamp).
        Messages can be multi-line, with a triplet per line. For each message that we can parse in this way,
        add the message 'ts' value to an acks array, which will be used to delete those messages from the 
        Slack channel to reduce the possibility of duplicate processing.

        For each triplet we can parse, send a new metric data point over a socket that we open to the Carbon
        daemon for this purpose, using newline-delimited plaintext.
        """
        messages = self.recv.get_messages()
        if len(messages) < 1:
            print "No metrics to report"
            return 0

        print "Sending stats..."

        acks = []
        metrics = []
        for m in messages:
            lines = [l.rstrip() for l in m['text'].splitlines()]
            for line in lines:
                parts = line.split(' ')
                if len(parts) < 3:
                    continue

                metrics.append("%s %s %s\n" % (parts[0], parts[1], parts[2]))
            acks.append(m['ts'])

        if len(acks) > 0:
            self.send.send_metrics(metrics)
            self.recv.ack_messages(acks)

        return len(acks)




#!/usr/bin/env python2

from setuptools import setup, find_packages
import sys

with open('README.rst', 'r') as f:
  long_description=f.read()

version='0.0.6'

setup(
    zip_safe=True,
    name='carbon-slack',
    version=version,
    description="Use Slack as a bus for GraphiteDB / Carbon",
    long_description=long_description,
    classifiers=[
      "Development Status :: 3 - Alpha",
      "Intended Audience :: Developers",
      "License :: OSI Approved :: GNU General Public License (GPL)",
      "Programming Language :: Python :: 2",
      "Topic :: Utilities",
    ],
    keywords='graphite carbon metrics monitoring slack',
    author='John Casey',
    author_email='jdcasey@commonjava.org',
    url='https://github.com/jdcasey/carbon-slack',
    license='GPLv3+',
    packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
    install_requires=[
      "slackclient",
      "click",
      "ruamel.yaml"
    ],
    test_suite="tests",
    entry_points={
      'console_scripts': [
        'carbon-slack-init = carbon_slack.command:init',
        'carbon-slack-recv = carbon_slack.command:recv',
        'carbon-slack-relay = carbon_slack.command:relay',
        'carbon-slack-send = carbon_slack.command:send'
      ],
    }
)




# This file is part of Carbon-Slack.
#
# Carbon-Slack is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Carbon-Slack is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Carbon-Slack.  If not, see <https://www.gnu.org/licenses/>.

# from carbon_slack.command import init, recv, relay, send

# __all__ = [
#     'init',
#     'recv',
#     'relay',
#     'send'
# ]